!-----------------------------------------------------------------------------
! Magic and Fight System --- Follower system
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: $id$
!  Contact author at:
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
! Grammar rules for MFS system and base verb routines
!------------------------------------------------------------------------------

!--------------------------------------------------------
! Grammar for knowledge (spell and tales)
!

Ifdef Knowledge;
Verb meta 'spiega' 'spg' 'racconta'
               * scope=KnownAbout				-> Explain;
Endif;

!--------------------------------------------------------
! Grammar for spells
!
Ifdef Spell;

Verb meta 'incantesimi' 'memoria' 'mem'
               *                               -> Spells;


Verb 'srotola' 'dispiega'
   'srotolo' 'dispiego'
               * held		                    -> Unfold;


!You can try to cast every spell you know...
Extend 'lancia' first
   * scope=KnownSpells				   -> Cast
   * scope=KnownSpells
      'a'/'al'/'allo'/'alla'/'agli'/'alle'/
      'su'/'sul'/'sui'/'sugli'/'sulla'/'sulle'
      noun -> Cast
   * scope=KnownSpells
      'su' 'di'                      noun -> Cast
   * scope=KnownSpells "sul" scope=KnownSpells			   -> Cast;

Verb 'direct cast'
               * 								->CastDirect
               * noun						->CastDirect
            * scope=KnownSpells 			->CastDirect;

Verb 'generator cast'
            *								->CastGenerator;


Verb 'scrivi'
   'scrivo'
      * scope=KnownSpells "su"/"sopra"/"nel"/"nello"/"nella"/
         "sul"/"sullo"/"sulla" held ->Scribe;

Verb 'studia' 'memorizza' 'impara'
   'studio' 'memorizzo' 'imparo'
               * scope=KnownSpells           -> Memorize;

Verb 'prepara'
   'preparo'
               * held                			-> Prepare;
Verb 'dimetti' 'riponi'
   'dimetto' 'ripongo'
               * held                			-> Repose;

Endif;

!-----------------------------------------------------------------------------------------
! WarMage Grammar Extensions/ Enemy combat system.
!-----------------------------------------------------------------------------------------
ifdef Warrior;

Verb meta 'diagnosi'
   *										    ->Diagnose
   * noun										->Diagnose;

Verb meta 'stats' 'statistiche'
   *											->PlaySheet
   * noun										->PlaySheet;


Verb 'impugna' 'sguaina' 'sfodera' 'brandisci'
   'impugno' 'sguaino' 'sfodero' 'brandisco'
      * held                     ->Draw;

Verb 'infodera' 'rinfodera' 'reinfodera'
   'infodero' 'rinfodero' 'reinfodero'
      * held                     ->Undraw;

#ifdef Spell;
Extend 'riponi' first
      * noun=Drawnable                ->Undraw
      * noun=Prepareable              ->Repose;
#endif;

#ifndef Spell;
Verb 'riponi'
   'ripongo'
      * noun=Drawnable                ->Undraw
      * noun=Prepareable              ->Repose;
#endif;

Extend 'attacca' first
      * noun=Damageable	               ->AttackDamageable;

Endif;

!--------------------------------------------------------
! Grammar definitions for follow system
!--------------------------------------------------------
Ifdef Follower;

Verb 'segui' 'seguite'
   'seguo'
      * noun								   ->Follow;

Verb 'seguimi' 'seguitemi'
   *                                   ->FollowMe;


Verb 'fermati' 'ferma' 'fermo' 'resta'
      *									   ->Stay
      * 'qui'							   ->Stay
      * 'qua'							   ->Stay;

Extend only 'stai' first
   * 'qui'                          ->Stay
   * 'qua'                          ->Stay;
Endif;
!--------------------------------------------------------
! Grammar definitions for time system
!--------------------------------------------------------

Ifdef MFS_Sky;

Ifdef MFS_SkyTime;
[SkyTimeSub;
   MFS_sky.tell_time();
];

[WaitUntilPeriodSub word period;
   wn = consult_from;
   word = NextWord();
   if ( word ) {
      period = MFS_sky.period_number( word );
      if ( period == -1 )
         "Mi spiace, ma non mi sembra che '", (address) word, "' sia un periodo del giorno.";
   }
   else {
      word = WordAddress( consult_from );
      "Mi spiace, ma non mi sembra che '", (byteword) word, "' sia un periodo del giorno.";
   }

   print "Il tempo scorre, mentre aspetti ", (string) MFS_sky.period_name( period ), ".^";
   MFS_sky.wait_period( period );

];

Verb meta "tempo"
   *					-> SkyTime;

Extend "aspetta"
   * "fino" "a"/"al"/"all^" topic -> WaitUntilPeriod;

Endif;

Ifdef MFS_ClockTime;

[ClockTimeSub;
   print "In questo momento sono le ore ";
   MFS_TimeScale.tell_time();
   print ".^";
];

Verb meta 'tempo'
   *					-> ClockTime;
Endif;

Ifdef MFS_ClockClock;

[ClockTimeSub;
   print "In questo momento sono le ore ";
   MFS_TimeScale.tell_time();
   print ".^";
];

Verb meta 'ora'
   *					-> ClockTime;
Endif;

[WaitHoursSub;
   if ( noun > 1) print "Trascorrono ", (LanguageNumber) noun, " ore";
   else print "Trascorre un ora";
   print "...^";
   if (noun == nothing) noun = 1;
   MFS_WaitTime( noun * 60 );
];

[WaitMinutesSub;
   if ( noun > 1) print "Trascorrono ", (LanguageNumber) noun, " minuti";
   else print "Trascorre un minuto";
   print "...^";
   MFS_WaitTime( noun );
];

[WaitUntilTimeSub;
   if ( noun < 0 || noun >23 || second < 0 || second > 60 )
      "@`E un ora impossibile, anche per te.";
   print "Attendi pazientemente fino alle ", noun, " e ", second, ".^";
   if ( the_time < noun * 60 + second )
      MFS_WaitTime( noun *60 + second - the_time);
   else {
      MFS_WaitTime( 1440 - the_time );
      MFS_WaitTime( noun *60 + second - ( 1440 - the_time) );
   }
];

Extend "aspetta"
   * number "ore"					-> WaitHours
   * "un^" "ora"					-> WaitHours
   * "un" "ora"					-> WaitHours
   * "una" "ora"					-> WaitHours
   * number "minuti"				-> WaitMinutes
   * "fino" "alle" number "e" number -> WaitUntilTime;

Endif;

!--------------------------------------------------------
! Grammar definitions for read system
!--------------------------------------------------------
Ifdef legible;
Extend 'leggi' first
      * legible						-> Read;
! could be replaced
[ ReadSub; <<Examine noun>>; ];
Endif;


! end fo MFS_Grammar.h
