!-----------------------------------------------------------------------------
! Magic and Fight System --- Time system based on sky
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Contact author at: antispam@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!  $Id: MFS_Sky.h,v 1.5 2003/10/26 01:29:21 gian Exp $
!-----------------------------------------------------------------------------
! Sistema di gestione del tempo fantasy
!------------------------------------------------------------------------------

Ifndef MFS_Sky;

!
! MFS_WaitTime ( number_of_minutes )
!

Global interrupt_wait = 0;
Global player_waiting = 0;

[MFS_WaitTime amount turns;
   interrupt_wait = 0;
   if ( amount == nothing || amount == 0 )
      amount = 1;
   if ( time_rate < 0 )
      turns = amount * ( 0 - time_rate );
   else
      turns = amount / time_rate;

   player_waiting = 1;
   while ( turns ) {
      TimePasses();
      #ifdef InformLibrary;
         InformLibrary.End_Turn_Sequence();
      #Ifnot;
         #ifdef EndTurnSequence;
            EndTurnSequence();
         #ifnot;
            Message fatalerror "waittime.h requires \
               InformLibrary.End_Turn_Sequence() or \
               EndTurnSequence() to be defined (this should be done \
               by the Inform Library)";
            break;
         #endif;
      #endif;
      !DisplayStatus();
      #IFV5;
      !DrawStatusLine();
      #endif;

      !waiting interrupt sequence
      if ( interrupt_wait ) {
         style bold;
         print "(La tua attesa @`e stata interrotta)^";
         style roman;
         interrupt_wait = 0;
         break;
      }

      turns --;
   }
   new_line;
   player_waiting = 0;
];

!
! Prints a string in clock format
!
[MFS_TellTime h m;
   h = the_time / 60;
   m = the_time % 60;
   if ( h < 10 ) print "0";
   print h,":";
   if ( m < 10 ) print "0";
   print m;
];

Attribute daylit;
Object MFS_Sky "cielo"
with
   name 'cielo' 'sole' 'stelle' 'luna',
   current_period -1,	! index in the period array
   period_vector
      6*60+45	! sunrise
      7*60	! dawn
      7*60+45	! morning
      12*60	! noon
      13*60	! afternoon
      17*60	! late afternoon
      19*60+15	! sunset
      19*60+30	! eveneeng
      20*60+30	! late evening
      21*60+15	! nigth
      23*60		! moonrise
      23*60+15	! moonnight
      4*60		! moonset
      4*60+15	! late nigth
      6*60, 	! before dawn
      ! total 15 periods per 1440 minutes ( 24 hours )
   sunset_period 6,		! Period after which the night begins (0 based!)

   description [;
      new_line;
      switch(self.current_period) {
         0: "Il sole sorge lontano ad est.";
         1: "La luce del nuovo giorno riempie il mondo.";
         2: "Il sole splende lucente sopra di te.";
         3: "Ora il sole @`e al suo zenit.";
         4: "Il sole, ancora alto, comincia la sua discesa.";
         5: "Il sole @`e ora basso sull'orizzonte.";
         6: "Il sole sta tramontando ad ovest.";
         7: "Il cielo sta scurendo, preparandosi per la notte.";
         8: "@`E notte. Le stelle brillano in un cielo terso.";
         9: "La notte si fa pi fredda, e le stelle brillano con maggiore intensit.";
         10: "Sta sorgendo la luna.";
         11: "La luna sta rischiarando la notte.";
         12: "La luna si approssima al tramonto.";
         13: "La luna @`e scesa sotto l'orizzonte, lasciando le stelle sole.";
         14: "Il cielo schiarisce, avvisando dell'arrivo di un nuovo giorno.";
      }
   ],

   ! can call it with no parameters or one parameter
   period_name[period pn;
      if (period == nothing) period = self.current_period;
      switch(period) {
         0: pn = "il sorgere del sole";
         1: pn = "l'alba";
         2: pn = "mattina";
         3: pn = "mezzogiorno";
         4: pn = "pomeriggio";
         5: pn = "tardo pomeriggio";
         6: pn = "il tramonto";
         7: pn = "sera";
         8: pn = "tarda sera";
         9: pn = "notte";
         10:pn = "il sorgere della luna";
         11:pn = "notte inoltrata";
         12:pn = "il tramonto della luna";
         13:pn = "tarda notte";
         14:pn = "l'aurora";
         default: pn = "?";
      }
      return pn;
   ],

   period_number[period;
      if (period == nothing) return self.current_period;

      switch(period) {
         'alba': return 1;
         'mattina', 'mattino': return 2;
         'mezzogiorno': return 3;
         'pomeriggio': return 4;
         'tramonto': return 6;
         'sera': return 7;
         'notte': return 9;
         'mezzanotte': return 11;
         'aurora': return 14;
      }
      return -1;
   ],

   is_night[time;
      if ( time == nothing ) time = the_time;
      if (self.calc_period( time ) > self.sunset_period )
         rtrue;
      rfalse;
   ],

   calc_period[time p1 p2;
      p1 = 0;
      p2 = 1;
      while (p1 < self.#period_vector/2) {
         ! Array roll
         if ( p2 == self.#period_vector/2 ) p2 = 0;
         ! day roll
         if ( self.&period_vector-->p1 > self.&period_vector-->p2 ) {
            if ( time >= self.&period_vector-->p1 ||
               time < self.&period_vector-->p2)
                  return p1;
         }
         else
            if ( time >= self.&period_vector-->p1 &&
               time < self.&period_vector-->p2)
                  return p1;
         p1++;
         p2++;
      }
      !impossible, if time is valid
      return 0;
   ],

   wait_period[ period mins;
      if ( period < 0 || period >= self.#period_vector/2 )
         rfalse;
      ! calculates time distance
      mins = self.&period_vector-->period - the_time;
      if ( mins < 0 )
         mins = 1440 + mins;
      MFS_WaitTime( mins );
   ],

   tell_time[ realtime;
      if (~~self.found_in()) {
         print "Non vedi il cielo; non puoi avere un'idea precisa, ma";
         realtime = the_time + random(180)-90;
      }
      else {
         realtime = the_time;
         print "Osservando il cielo capisci che";
      }

      print " dovrebbe essere ", (string)
         self.period_name( self.calc_period(realtime) ), ".^";
   ],

   found_in [;
      if (location has daylit)
         rtrue;
      rfalse;
   ],

   daemon [p;
      ! initialization, first time is run
      if ( self.current_period < 0 )  {
         self.current_period = self.calc_period( the_time );
         return;
      }

      p = self.calc_period( the_time );
      if (p ~= self.current_period) {
         self.current_period = p;
         if ( self.found_in() )
            self.description();
      }
   ],
   has scenery concealed;


Endif; ! ifndef sky

! end of MFS_Time.h
