!-----------------------------------------------------------------------------
! Magic And Fight System --- Speech, a general topic parsing routine
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.0
!  Last Revised:
!  Contact author at: antispam@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
!
!-----------------------------------------------------

Ifndef MFS_Speech;

Ifndef WORDSIZE;
    Constant WORDSIZE 2;
Endif;

Include "MFS_BasicWords.h";

[ MFS_SpeechNextRule object rule pos;
   while ( pos < object.#rule/WORDSIZE && object.&rule-->pos ~= 0 ) pos++;
   ! skip rule action
   pos++;
   pos++;
   return pos;
];

[ MFS_SpeechFollowRule word object rule;
   if ( word == 0 ) { ! done. If this is a terminal rule, we are ok
      if( metaclass(rule) == String ) {
         print (string) rule,"^";
      }
      else if ( metaclass(rule) == Routine ) {
         rule( object );
      }
      else {
         ! unlucky. This is NOT a terminal rule, but a rule
         ! referencing something...
         if ( metaclass( rule ) == 2 ) !  but...
         {
            ! is it a routine property ?...
            if ( metaclass(object.rule) == Routine )
            {
               object.rule( object );
            }
            ! is it a string property ?...
            else if ( metaclass(object.rule) == String
                  && object.#rule/WORDSIZE == 1 )
            {
                print (string) object.rule,"^";
            }
            else ! no way, we have to give up; it is really nonterminal
               return false;
         }
         else
            return false; ! it is nothing, or something weird.
      }
   }
   else { !more word. Scan THIS rule and continue.
      if( metaclass(rule) == String or Routine )
         return false; ! was a terminal rule...
      else {
         wn --;  ! re-read last (that brought us here)
         return MFS_SpeechScanRule( object, rule );
      }
   }

   ! sucessfull
   rtrue;
];

[ MFS_SpeechScanRule object rule
   pos wordpos word;

   ! save current word position
   wordPos = wn;
   pos = 0;
   while( pos < object.#rule/WORDSIZE  ) {
      wn= wordPos;
      word = MFS_GetWord( 1 ); ! do not restart

      while( pos < object.#rule/WORDSIZE ) {

         ! forced continuation rule?
         if ( object.&rule-->pos == '-' ) {
            pos ++;
            if ( MFS_SpeechFollowRule( word, object, object.&rule-->pos ) )
               rtrue;
            wn = wordPos; ! because we must go back to re-read last word
            ! if the child rule fails, try another one
            ! will be on next rule at the first pos++
         }
         else if ( object.&rule-->pos == 0 ) {
            !FOUND: follow this rule
            pos ++;
            if ( pos == object.#rule/WORDSIZE )
               "[MFS_Speech PROGRAMMING ERROR: No rule for this object
               speech at pos ", pos,"]";

            if ( MFS_SpeechFollowRule( word, object, object.&rule-->pos ) )
               rtrue;
            wn = wordPos; ! because we must go back to re-read last word

            ! if the child rule fails, try another one
            ! will be on next rule at the first pos++
         }
         else if ( object.&rule-->pos ~= '*' && word ~= object.&rule-->pos ) ! valid also if word == 0
         {
            ! if this rule fails, try another one
            break;
         }
         ! Match goes on...
         pos++;
         word = MFS_GetWord( 1 ); ! do not restart
      }

      if ( pos < object.#rule/WORDSIZE ) { ! done with rules?
         ! no: this rule does not matches
         pos = MFS_SpeechNextRule( object, rule, pos );
      }
   }

   rfalse;
];

[MFS_Speech spc prop;

   wn = consult_from;

   ! find a rule matching first word.
   return MFS_SpeechScanRule( spc, prop );

];

Endif;
