!-----------------------------------------------------------------------------
! Magic And Fight System --- Combact system
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at: giancarlo@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
! Defines warrior class, weapon, armor and bonus giver in combact
!-----------------------------------------------------------------------------

Ifndef Warrior;

ifndef damage;
   include "MVS_damage";
endif;
ifndef pron;
   include "MVS_pronouns";
endif;

Attribute healing;
!-----------------------------------------------------------------
! System constant
!
Constant MVSW_VERB_HI 1; ! lots of comments in battle
Constant MVSW_VERB_LO 0; ! few comments in battle

Constant MVSW_HELMET	1;  ! Armor to be wear as an helmet
Constant MVSW_ARMOR 	2;  ! Armor to be wear as a dress
Constant MVSW_SHIELD	3;  ! Armor to be wear as a shield
Constant MVSW_BOOTS 	4;  ! Armor to be wear as boots
Constant MVSW_GIRDLE	5;  ! Armor to be wear as a girdle


!-----------------------------------------------------------------
! If an object has hostile it will attack any nearby enemy
!
Attribute hostile;

!-------------------------------------------------------------------
! If an area has fight_arena, the fight_referee will take care of it
!
Attribute fight_arena;

!----------------------------------------------------------------------
! If this property is a routine returning true, the object will provide
! a bonus for the owner.
!
Ifndef giving_bonus;
   Property giving_bonus;
Endif;


!-------------------------------------------------------------------
! Basic warrior. Player can't be a basic warrior.
!
class Warrior
private
      resolve_fight [ enemy min max value;

         if (enemy provides defend)	{
            value = random(10)-1;

            if (value+self.offend() <= enemy.defend())
               return 0;
         }
         value = max - min;
         if(value < 0) value = -value;
         if(value > 0) value = random(value+1)-1;
         return value+min+
            GlobalValue(self,base_damage,giving_bonus);
   ],

   filter_source[;
      rfalse;
   ],

   display[strnum target;
      if (self.mute) return;

      switch(strnum) {
         0: if (target == player) {
               print "^%%-> ",(The) self, " ti ";
               self.attack_string();
            }
            else {
               if ( self == player )
                  print "^::-> Tu ";
               else
                  print "^&&-> ", (The) self, " ";
               self.attack_string();
               print " ", (arta) target;
            }
            return;

         1: print "... ma ";
            self.defense_string( target );
            rtrue;

         2: if (target == player) "... ma la tua armatura resiste!";
            "... ma l'armatura ", (artdi) target, " resiste!";
         3: if (target == player)
               "!!! E ", (the) self, " ti colpisce!";
            if (self == player )
               "!!! E colpisci!";
            "!!! E ", (the) self, " colpisce ",(the) target,"!";
         4: if (target == player)
               "... Ma sei immune all'attacco!^";
            "... Ma ", (the) target, " @`e immune all'attacco!^";
         5: if ( self == player ) "Le tue statistiche di gioco:^";
            if (self.verbosity == MVSW_VERB_LO)
               "Non ne puoi essere certo.";
               "Statistiche di gioco ", (artdi) self, ":^";
         6: if ( self == player )
               print "Al momento, la tua salute @`e ";
            else
               print "La salute ", (artdi) self," sembra essere ";
            return;
         7: if ( self == player ) {
               style bold;
               print (string) random( "Sopporti ", "Subisci ");
            }
            else
               print (The) self, (string) random(" sopporta ", " subisce ");
            return;
         8: if ( self == player ) {
               style bold;
               print "La tua salute migliora";
            }
            else
               print "La salute ", (artdi) self, "@`e ristorata";
            return;

         9: if ( self == player ) {
               print "Sei";
            }
            else
               print (the) self, " ",(isorare) self;
            return;
      }

   ],

   attack_string[fem;

      fem = random(2);
      switch(fem) {
         1: print "assest";
         2: print "piazz";
      }
      if ( self == player ) print "i";
      else print "a";

      fem = random(2);
      print " un";
      if ( fem == 1 ) print "a";

      print " ";
      if (random(10) <= 3) {
         if ( fem == 1 )
            print (string) random(
               "forte",
               "pericolosa",
               "vigorosa",
               "precisa");
         else
            print (string) random(
               "forte",
               "pericoloso",
               "vigoroso",
               "preciso");
         print " ";
      }


      if ( fem == 1)
         print (string) random(
            "pennellata",
            "sferzata",
            "stoccata",
            "spazzata"
         );
      else
         print (string) random(
            "attacco",
            "colpo",
            "affondo",
            "fendente"
         );

      rtrue;
   ],

   defense_string[ target;

      if ( random(10) < 4 )
         print (string) random (
            "in qualche modo ",
            "fortunosamente ",
            "agevolmente ",
            "con eleganza "
         );

      if ( target == player )
         print (string) random (
            "riesci a",
            "sei capace di",
            "sei in grado di");
      else
         print (the) target, " ", (string) random (
            "riesce a",
            "@`e in grado di",
            "@`e capace di");
      print " ";

      print (string) random(
         "parare",
         "evadere",
         "sostenere",
         "evitare"
      );


      print " ";
      if ( random(10) < 5 ) {
         print (string) random (
            "il colpo",
            "l'assalto",
            "l'attacco",
            "l'affondo");
         print " ";
         if (random(10) <=3) {
            print (string) random (
               "esperto",
               "assassino",
               "pericoloso",
               "insidioso",
               "inusuale");
            print " ";
         }
      }
      else {
         print "la stoccata";
         print " ";
         if (random(10) <=3) {
            print (string) random (
               "esperta",
               "assassina",
               "pericolosa",
               "insidiosa");
            print " ";
         }
      }

      if ( random(10) > 7 ) {
         ".";
      }

      print (string) random(" con una ", " grazie ad una ", " sfoderando una ");
      if (random(10) <=5) {
         print (string) random (
            "agile",
            "rapida",
            "intelligente",
            "veloce",
            "buona",
            "eccellente",
            "inaspettata",
            "ottima");
      }
      print " ";
      print (string) random (
         "schivata",
         "mossa",
         "manovra");
      ".";
   ],


with
   alliance 0,

   max_health 10,
   health 10,

   base_damage 0,
   attack 1,
   defense 1,
   armor 0,
   initiative 0,

   min_damage 1,
   max_damage 2,
   will_attack 0,

   initiative_roll 0,

   verbosity MVSW_VERB_HI,		! see enemy stats...

   ! nella versione italiana, le mani nude devono contenere
   ! tutta l'interlocuzione.
   hands_name "a mani nude",
   mute 0,

   find_weapon [ wp;
      objectloop (wp in self &&
         wp ofclass Weapon)
      if (wp.drawn == 1) return wp;
      return nothing;
   ],

   attack_enemy[tenemy;
      self.will_attack = tenemy;
      self.initiative_roll = GlobalValue(self, initiative,
         giving_bonus)+random(10);
      give parent(self) fight_arena;
      if (TestScope(self)) self.mute = 0;
      else self.mute = 1;
   ],

   complete_attack[tenemy weapon fpoint min max dmgt;
      if (tenemy == nothing) tenemy = self.will_attack;
      self.display(0,tenemy);

      weapon = self.find_weapon();
      if (weapon == nothing) {
         ! nella versione italiana, le mani nude devono contenere
         ! tutta l'interlocuzione.
         if (~~self.mute) print " ", (string)self.hands_name;
         dmgt = DMG_BAREHANDS;
         min = self.min_damage;
         max = self.max_damage;
      }
      else {
         if (~~self.mute) {
            print (string) random( " con ", " usando " );
            print (the) weapon;
         }
         min = weapon.min_damage;
         max = weapon.max_damage;
         dmgt = weapon.dmg_type;
      }
      if (~~self.mute) {
         print "...^";
      }

      fpoint = self.resolve_fight(tenemy,min,max);

      if (fpoint == 0) {
         self.display(1,tenemy);
         rfalse;
      }

      fpoint = fpoint - GlobalValue(tenemy,armor,giving_bonus);

      if (fpoint <= 0) {
         self.display(2,tenemy);
         rfalse;
      }

      ! if the program reaches here, the blow has hit
      self.display(3,tenemy);

      if (tenemy.damage(fpoint, dmgt, self ) == 0) {
         self.display(4,tenemy);
      }
      rtrue; ! an attack has been executed
   ],

   dead_proc [;
      if (TestScope(self)) {
         print (The) self, " muore con un terribile urlo di agonia";
         if (child(self) ~= nothing)
            print", lasciando cadere ci@`o che portava con se'.";
         print "!^^";
      }

      self.self_remove();
   ],

   self_remove [ obj;
      while ( (obj = child(self) )~= nothing)
         move obj to parent(self);
      remove self;
   ],


   diagnose [value;
      self.display(6);

      value = (self.health*100)/self.max_health;
      switch(value) {
         100:  "al massimo.";
         90 to 99: "molto alta.";
         80 to 89: "piuttosto alta.";
         60 to 79: "buona.";
         30 to 59: "un po' preoccupante.";
         10 to 29: "molto preoccupante.";
         default: "drammaticamente bassa!";
      }
   ],

   play_sheet [ weapon;
      self.display(5);
      if (self ~= player && self.verbosity == MVSW_VERB_LO)
         return;
      print "Attacco: ",GlobalValue(self,attack,giving_bonus),
      "^Difesa: ",GlobalValue(self,defense,giving_bonus),"^";
      print "Danno: da ",self.min_damage, " a ",
         self.max_damage,"^";
      print "Danno base: ",
         GlobalValue(self,base_damage,giving_bonus);
      print "^Armatura: ",GlobalValue(self,armor,giving_bonus),"^";
      print "Velocit@`a: ",GlobalValue(self,initiative,giving_bonus),"^";
      print "Salute: ", self.health, " su ",
         self.max_health, "^";

      weapon = self.find_weapon();

      if (weapon ~= nothing) {
         print "In questo momento, sta";
         if ( self == player ) print "i";

         print " brandendo ", (the) weapon,
            " pront", (gan) weapon, " a combattere.^";
         print "Bonus in attacco: ", weapon.attack,
            "^Bonus in difesa: ",weapon.defense,
            "^Danno: da ",weapon.min_damage,
            " a ",weapon.max_damage,"^";
      }
   ],

   ! to be overloaded: filter_source

   damage [value source fem;
      if (self.filter_source(source,sender)) rfalse;
      if (value <= 0 ) rtrue;

      self.health = self.health - value;

      if (self.health <= 0) {
         self.dead_proc();
         rtrue;
      }

      if (self.mute) rtrue;
      if ( source == DMG_FATIGUE && self ~= player ) rtrue;

      self.display(7);

      value = value*100/self.max_health;
      if ( source == DMG_FATIGUE) {
         print "uno shock";
         fem = "o";
      }
      else {
         if ( random(2) == 1 ) {
            print (string) random( "uno shock", "un danno" );
            fem = "o";
         }
         else {
            print "una ferita";
            fem = "a";
         }
      }
      switch (value) {
         0,1,2: print " risibile";
         3 to 10: print " minim", (string) fem;
         11 to 20: print " molto piccol", (string) fem;
         21 to 30: print " piccol", (string) fem;
         31 to 40: print " rilevante";
         41 to 50: print " seri", (string) fem;
         51 to 60: print " molto seri", (string) fem;
         61 to 70: print " pericolos", (string) fem;
         71 to 80: print " pericolosissim", (string) fem;
         81 to 99: print " quasi mortale";
         default: if (value >= 100)  print " mortale";
            else print " sotto zero";
      }
      print "!^";
      style roman;
      rtrue;
   ],

   heal [value;
      self.display(8);
      if ( value == nothing )
         value = 1;
      self.health = self.health + value;
      if (self.health >= self.max_health) {
         self.health = self.max_health;
         if (~~self.mute)
            print ", ed @`e ripristinata al suo massimo!";
      }
      if (~~self.mute) print "!^";
      style roman;
   ],

   defend [;
      return GlobalValue(self,defense,giving_bonus);
   ],

   offend [;
      return GlobalValue(self,attack,giving_bonus);
   ];


!-------------------------------------------------------------------
! Player warrior.
!
Class PC_Warrior class Warrior
   with

   max_thirst 100,			! maximum allowed of ...
   max_hungry 100,
   max_tire 100,
   thirst 0,				! ... thirst level
   hungry 0,				! ... hungry level
   tire 0,					! ... sleepless time.

   level_gained[att def bd;
      self.attack = self.attack+att;
      self.defense = self.defense+def;
      self.base_damage = self.base_damage+bd;
      style bold;
      if (att>0)
         print "Il tuo attacco @`e migliorato.^";
      if (def>0)
         print "La tua difesa @`e migliorata.^";
      if (bd>0)
         print "Il tuo danno base @`e migliorato.^";
      style roman;
   ],

   complete_attack[tenemy;
      if (tenemy == nothing) tenemy = self.will_attack;
      if (parent (tenemy) == nothing)
         "Il bersaglio che avevi designato non @`e pi@`u qui.^";
      return self.Warrior::complete_attack(tenemy);
   ],

   dead_proc [;
      print "^^   Il tuo corpo e la tua volont@`a non sono in grado di sostenere questo danno.
      La vita scivola via dalle tue dita, e per guanto tu provi a trattenerla, ogni cosa
      sparisce dalla tua vista...^^";
      deadflag = 1;
   ],

   attack_enemy [tenemy;
      if (tenemy provides alliance &&
         tenemy.alliance == self.alliance)
         "Ma ", (the) tenemy, " tuo alleato, non tuo nemico!";
      self.warrior::attack_enemy(tenemy);
   ];

!-------------------------------------------------------------------
! Warrior for NPC characters.
!
class NPC_Warrior class Warrior
private
   wait 1,			! turns to wait before becaming active
   aggr_rate 1,	! I will attack every n turns ...
   aggressivity 100, ! ... with a certain %

   waited 0,		! flags
   aggr_turns 0,

   wait_message [;
   ],

   !calc_attack will attack an opponent
   !if a certain number of turns have passed, and only
   !if the aggressivity check is passed.
   !calc_attack is called only if the enemy and the
   !player are in the same room

   calc_attack [;
      if (self.aggr_rate == 0) return 0;

      self.aggr_turns++;
      if (self.aggr_turns >= self.aggr_rate){  !time to figth
            self.aggr_turns = 0;
            if (random(100) <= self.aggressivity) { ! I will attack
               return 1;
            }
      }
      return 0;
   ],

with

   !rfalse ==  we don't want to attack
   !rtrue == we have a target, and we want to attack
   chose_attack [ enemy count obj;
      if (self.will_attack ~= nothing) rtrue;
      ! test if there are one or more opponents

      count = 0;
      objectloop( obj in parent(self) && obj has animate &&
               obj provides damage && obj.alliance ~= self.alliance)
         count++;

      if (count == 0) {
         self.waited = 0;
         rfalse;
      }

      if (self.waited < self.wait) {
            self.waited++;
            if (TestScope(self))
            self.wait_message();
      }
      else {
         if (self.calc_attack() == 0) rfalse;
         count = random(count);

         objectloop( obj in parent(self) &&
                  obj has animate && obj provides damage &&
                  obj.alliance ~= self.alliance) {
            count--;
            if (count == 0) {
               enemy = obj;
               break;
            }
         }
         self.attack_enemy(enemy);
         rtrue;
      }
   ],

   complete_attack[tenemy;
      if (tenemy == nothing) tenemy = self.will_attack;
      !killed meanwhile?
      if (parent(tenemy) == nothing)
         if (~~self.mute) {
            print (The) (self) ," non pu@`o pi@`u attaccare ",
               (the) tenemy, ".^^";
            return;
         }

      self.Warrior::complete_attack( tenemy );
   ],

   diagnose [;
      self.Warrior::diagnose();
      if (self.verbosity == MVSW_VERB_LO) return;

      print (The) self, " sembra essere ";
      switch(self.health) {
         0 to 5: "facilmente battibile.^";
         5 to 15: "un buon avversario.^";
         15 to 30: "piuttosto forte.^";
         30 to 50: "molto forte.^";
         default: "quasi immortale.^";
      }
   ],

   defend[;
      ! No more waiting before attack
      self.waited = self.wait;
      return self.Warrior::defend();
   ],

   react_after[;

      AttackDamageable:
         if (self.will_attack ~= nothing)
            rfalse; !already doing

         if (actor == player) {
            if (player provides alliance &&
               player.alliance == self.alliance){
               if ( (~~(noun provides alliance)) ||
                  noun.alliance ~= self.alliance)
                  self.will_attack = noun;
            }
         }

      rfalse; ! other actions may have something to do.
   ],

   orders [;
      AttackDamageable:
         if (player.alliance ~= self.alliance) {
            print (The) self, " non ti ascolter@`a!^";
            rfalse;
         }
         if (noun provides alliance) {
            if (noun.alliance == self.alliance)
               print (The) self, " orgogliosamente si rifiuta di attaccare un suo compagno.^";
            else {
               actor = self;
               <<AttackDamageable noun>>;
            }
            rtrue;
         }
         else {
            actor = self;
            <<AttackDamageable noun>>;
         }
   ],
   has animate;


!--------------------------------------------------------------
! Basic weapon ( that can be also a bonus giver)
!
class Weapon
with
   min_damage 1,
   max_damage 5,
   attack 1,
   defense 1,
   drawn 0,
   dmg_type DMG_CUTWEAPON,

   giving_bonus [;
      if (self.drawn > 0) rtrue;
      rfalse;
   ],


   draw [;
      if (self.drawn == 0) {
         self.drawn = 1;
         if ( actor == player )
            "Estrai ",(the) self, " brandendol", (gan) self, " di fronte a te.";
      }
      if ( actor == player )
         "Ma ", (the) self, " @`e gi@`a brandit", (gan) self, "!";
      rtrue;
   ],

   undraw [;
      if (self.drawn == 1) {
         self.drawn = 0;
         if ( actor == player )
            "Riponi ", (the) self, " sentendoti indifeso come un bambino.";
      }
      if ( actor == player )
         "Ma stai brandendo ", (the) self, " (per ora...)";
      rtrue;
   ],

   invent [;
      if (inventory_stage == 1) rfalse;
      if (self.drawn == 0) {
         print " (ripost", (gan) self, ")";
         rfalse;
      }
      print " (impugnat", (gan) self, "!)";
      rfalse;
   ],

   before [;
      Take, Drop: self.drawn = 0;
   ],

   after [;
      Examine:
         DescBonusGiver(self);
   ];

!--------------------------------------------------------------
! Armor that can be worn
!
class ClothingArmor
with
   armor 1,
   type,

   giving_bonus [;
      if (self has worn) rtrue;
      rfalse;
   ],

   before [obj ;
      Wear:
         if (self has worn)
            "Stai gi@`a indossando ", (the) self;

         objectloop(obj in parent(self) &&
            obj ofclass ClothingArmor &&
            obj has worn)
               if (obj ~= self &&
                  obj.type == self.type) {
                  print "(prima togliendosi ",
                     (the) obj,")^";
                  <Disrobe obj>;
                  if (obj has worn) rtrue;
               }
         give self worn;
         "Ora ti senti pi@`u protetto.";
   ],

   after [;
      Examine:
         DescBonusGiver(self);
   ],

   has clothing;


!-----------------------------------------------------------------
! The fight referee takes care of handle all potential and actual
! fights being disputed in the game.
! To initialize the fight referee you hage to call
!         StartDaemon(Fight_Referee)
! from your program
!
Object FightReferee
with
   daemon [room war done max;
      ! decide to fight loop
      objectloop(war has hostile) {
         room = parent(war);
         if (room ~= nothing  && room hasnt absent)
            if (war ofclass NPC_Warrior)
               war.chose_attack();
      }

      ! resolve fight loop
      objectloop(room has fight_arena) {
         done = true;
         while (done) {
            done = false;
            max = nothing;
            objectloop( war in room && war ofclass Warrior) {
               if (room has absent)
                  war.will_attack = nothing;
               else if (war.will_attack ~= nothing) {
                  if (max == nothing)
                     max = war;
                  if (max.initiative_roll <
                     war.initiative_roll)
                     max = war;
                  done = true;
               }
            }

            if (max ~=nothing) {
               max.complete_attack();
               max.will_attack = nothing;
            }
            ! game over?
            if ( deadflag == 1 ) break;
         }
         give room ~fight_arena;
      }

      !healing monster loop
      objectloop(war provides heal)
         if ( war has healing )
            war.heal();
   ];


[Damageable;
	if (noun provides damage && TestScope(noun))
		return true;
	return false;
];

[Drawnable;
	if (noun provides draw && noun provides undraw && noun
		in actor) rtrue;
	rfalse;
];

[DrawSub;
	if (~~ (noun provides draw))
		"Affrontare un nemico con un", (gan) noun," ", (name) noun, " sarebbe inutile.";

	noun.draw();
	return AfterRoutines();
];

[UndrawSub;
	if (~~ (noun provides undraw))
		"Non sai nemmeno come brandirl", (gan) noun,", figuriamoci infoderarl", (gan) noun,"!";
	noun.undraw();
	return AfterRoutines();
];

[DiagnoseSub;
	if (noun == 0) ! For the player
		if (actor provides diagnose)
			actor.diagnose();
		else
			if (actor == player)
				"Nel tuo stato attuale, non dovresti preoccuparti della tua salute.";
			else
				"Cosa?";

	else
		if (noun provides diagnose)
			noun.diagnose();
		else
			"Non ci sono segni visibili che ti aiutino a intuire lo stato di salute ",
				(the) noun,".";
];

[PlaySheetSub obj;
	if (noun == 0)
		obj = actor;
	else
		obj = noun;

	if (obj provides play_sheet) {
		obj.play_sheet();
	}
	else {
		if (obj == player) {
			"Nessuna scheda del giocatore. Il tuo master deve averla persa!^";
		}
		else
			print (The) obj, " non ha statistiche disponibili.^";
	}
];

[AttackDamageableSub;
	!attack only if the actor can attack
	if (~~(player provides attack_enemy) )
			"Nella tua condizione sarebbe meglio tentare qualche altro modo.";

	!else if (TestScope(player,actor))
	!	print (The) actor, " non @`e in grado di usare una simile violenza.";

	!let's give a defence chance

	if (location has absent)
		"Questo @`e un luogo in cui la violenza non @`e consentita.";

	actor.attack_enemy(noun);

	! group 2 action
	return AfterRoutines();
];

Endif;

! end of MFS_War.h
