!-----------------------------------------------------------------------------
! Magic And Fight System --- Combact system
! For INFORM compiler and Z-Machine text based Adventure games
!-----------------------------------------------------------------------------
!  Author: Giancarlo Niccolai
!  Version: 1.3
!  Last Revised: 2002-11-24
!  Contact author at: giancarlo@niccolai.ws
!  Feel free to use and distribute this file, but please report me if
!  you intend to change it (in better), so we can upgrade together
!
!  Released under GPL 2.0, or any later version at your will
!
!-----------------------------------------------------------------------------
! Sortcut to define very complicated NPC and player objects, such as mages
! capable to fight.
!-----------------------------------------------------------------------------

Ifndef WarMage;

Ifndef Warrior;
   include "MVS_War.h";
Endif;
Ifndef NPC_Memory;
   include "MVS_Magic.h";
Endif;

!-------------------------------------------------------
! A warmage that can take care of itself
!

class NPC_WarMage class NPC_Warrior
with
   memory,
   cast_rate 50,
   can_attack true,

   chose_attack [;
      ! No new target while casting spells
      if ( self.memory.timeLeft > 0 ) {
         self.will_attack = nothing;
         rfalse;
      }
      return self.NPC_Warrior::chose_attack();
   ],

   complete_attack [ enemy sp;
      if (enemy == nothing) enemy = self.will_attack;

      if (random(100) <= self.cast_rate) {
         sp = self.memory.pick_spell();
         if (sp ~= nothing) {
            if (sp provides fire)
               sp = self.memory.start_cast(sp,enemy);
            else
               sp = self.memory.start_cast(sp);
         }
      }
      if ((sp == false || sp == nothing)
         && self.can_attack)
         self.NPC_Warrior::complete_attack(enemy);
   ],

   damage[ dam type;

      if ( self.memory.timeLeft > 0) {
         if ( random(2) == 1 ) {
            print "Questo colpo riesce ad interrompere la concentrazione ",
               (artdi) self, "!^";
         }
         else {
            print "La concentrazione ", (artdi) self, " @`e interrotta!^";

         }
         StopDaemon( self.memory );
         self.memory.timeLeft = 0;
      }
      return self.NPC_Warrior::damage( dam, type );
   ];


!------------------------------------------------------------
! A warrior that is also a mage (useable for player)
!
class WarMage class PC_Warrior
with
   memory,

   level_up_points 10,
   next_level_up 10,

   level_gained[att def bd magic;
      self.memory.mage_power =
         self.memory.mage_power + magic;
      style bold;
      if (magic > 0) {
         print "Il tuo potere magico @`e aumentato!^";
      }
      self.PC_Warrior::level_gained(att, def, bd);
      style roman;
   ],

   damage[ dam type;
      if ( self.memory.timeLeft > 0) {
         style bold;
         print (string) random(
            "Questo colpo ha interrotto la tua concentrazione!^",
            "La tua concentrazione @`e interrotta!^");
         style roman;
         self.memory.spellToFire = 0;
         self.memory.timeLeft = 0;
         StopDaemon( self.memory );
      }
      return self.PC_Warrior::damage( dam, type );
   ],

   play_sheet [;
      self.PC_Warrior::play_sheet();
      new_line;
      print "Per quanto riguarda la magia, queste sono le tue attuali abilit@`a:^";
      print "Potere: ", self.memory.mage_power + GlobalValue( self, power_bonus, giving_bonus ), "^";
      if ( ~ (self.memory ofclass CFMemory ) )
         print "Bonus su ", (string) self.memory.cost_name, ": ", self.memory.cost_bonus +
            GlobalValue( self, cost_bonus, giving_bonus ), "^";
      if ( self.memory.timed_spells == MFS_TIMED )
         print "Bonus sul tempo di lancio: ", self.memory.time_bonus + GlobalValue( self, time_bonus, giving_bonus ), "^";

      if ( self.memory ofclass MPMemory )
         print (string) self.memory.cost_name, ": ",
            self.memory.mpoints, " su ", self.memory.max_mpoints, "^";
   ],

   each_turn [;
      if (score >= self.next_level_up && deadflag == 0) {
         print "^Dopo questo sforzo, hai compreso i tuoi poteri meglio di prima.^";
         self.level_gained(random(1,2), random(1,2), random(1,2), 1 );
         self.max_health = self.max_health + 5 + random(5);
         self.next_level_up = self.next_level_up + self.level_up_points;
         self.heal(self.max_health-self.health);
      }
      rfalse;
   ];


Endif;

! end of MFS_WarMage.h
